@@ -9,7 +9,7 @@ from rdflib import Graph, RDF, URIRef, util
 from rdflib.namespace import split_uri
 import validators
 import prov.model as pm
-from prov.model import QualifiedName
+from prov.model import QualifiedName, Identifier
 from prov.model import Namespace as provNamespace
 import requests
 from rapidfuzz import fuzz
@@ -58,8 +58,8 @@ from cognitiveatlas.api import get_concept, get_disorder
 
 
 # set if we're running in production or testing mode
-INTERLEX_MODE = 'test'
-#INTERLEX_MODE = 'production'
+#INTERLEX_MODE = 'test'
+INTERLEX_MODE = 'production'
 if INTERLEX_MODE == 'test':
     INTERLEX_PREFIX = 'tmp_'
     #INTERLEX_ENDPOINT = "https://beta.scicrunch.org/api/1/"
@@ -113,7 +113,7 @@ def read_nidm(nidmDoc):
             # add namespaces to prov graph
             for name, namespace in rdf_graph_parse.namespaces():
                 # skip these default namespaces in prov Document
-                if (name != 'prov') and (name != 'xsd') and (name != 'nidm'):
+                if (name != 'prov') and (name != 'xsd') and (name != 'nidm') and (name != 'niiri'):
                     project.graph.add_namespace(name, namespace)
 
         else:
@@ -128,7 +128,7 @@ def read_nidm(nidmDoc):
         #add namespaces to prov graph
         for name, namespace in rdf_graph_parse.namespaces():
             #skip these default namespaces in prov Document
-            if (name != 'prov') and (name != 'xsd') and (name != 'nidm'):
+            if (name != 'prov') and (name != 'xsd') and (name != 'nidm') and (name != 'niiri'):
                 project.graph.add_namespace(name, namespace)
 
         #Cycle through Project metadata adding to prov graph
@@ -168,7 +168,7 @@ def read_nidm(nidmDoc):
                 if str(rdf_type) == Constants.NIDM_ACQUISITION_ACTIVITY.uri:
                     #first find the entity generated by this acquisition activity
                     for acq_obj in rdf_graph_parse.subjects(predicate=Constants.PROV["wasGeneratedBy"],object=acq):
-                        #Split subject URI for session into namespace, uuid
+                        #Split subject URI for acquisition object (entity) into namespace, uuid
                         nm,acq_obj_uuid = split_uri(acq_obj)
                         #print("acquisition object uuid: %s" %acq_obj_uuid)
 
@@ -369,27 +369,32 @@ def add_metadata_for_subject (rdf_graph,subject_uri,namespaces,nidm_obj):
             found_uri = find_in_namespaces(search_uri=URIRef(obj_nm), namespaces=namespaces)
             # if obj_nm is not in namespaces then it must just be part of some URI in the triple
             # so just add it as a prov.Identifier
-            if not found_uri:
+            if (not found_uri) and (obj_nm != Constants.PROV) and (obj_nm != Constants.XSD):
                 predicate = pm.QualifiedName(namespace=Namespace(str(predicate)), localpart="")
             # else add as explicit prov.QualifiedName because it's easier to read
-            else:
-                predicate = pm.QualifiedName(namespace=Namespace(obj_nm),localpart=obj_term)
+            #else:
+            #    predicate = Identifier(predicate)
             if (validators.url(objects)) and (predicate != Constants.PROV['Location']):
                 # try to split the URI to namespace and local parts, if fails just use the entire URI.
                 try:
                     #create qualified names for objects
                     obj_nm,obj_term = split_uri(objects)
+
+                    # added because PyNIDM agent, activity, and entity classes already add the type
+                    if ((objects == Constants.PROV['Activity']) or (objects == Constants.PROV['Agent']) or
+                        (objects == Constants.PROV['Entity'])):
+                        continue
                     # special case if obj_nm is prov, xsd, or nidm namespaces.  These are added
                     # automatically by provDocument so they aren't accessible via the namespaces list
                     # so we check explicitly here
                     if ((obj_nm == str(Constants.PROV))):
-                        nidm_obj.add_attributes({predicate: pm.QualifiedName(Namespace(obj_nm), obj_term)})
+                        nidm_obj.add_attributes({predicate: Constants.PROV[obj_term]})
                     else:
                         found_uri = find_in_namespaces(search_uri=URIRef(obj_nm),namespaces=namespaces)
                         # if obj_nm is not in namespaces then it must just be part of some URI in the triple
                         # so just add it as a prov.Identifier
                         if not found_uri:
-                            nidm_obj.add_attributes({predicate: pm.QualifiedName(namespace=Namespace(str(objects)),localpart="")})
+                            nidm_obj.add_attributes({predicate: Identifier(objects)})
                         # else add as explicit prov.QualifiedName because it's easier to read
                         else:
                             nidm_obj.add_attributes({predicate: pm.QualifiedName(found_uri, obj_term)})
@@ -400,7 +405,7 @@ def add_metadata_for_subject (rdf_graph,subject_uri,namespaces,nidm_obj):
                 # check if objects is a url and if so store it as a URIRef else a Literal
                 if validators.url(objects):
                     obj_nm, obj_term = split_uri(objects)
-                    nidm_obj.add_attributes({predicate : pm.QualifiedName(namespace=Namespace(obj_nm),localpart=obj_term)})
+                    nidm_obj.add_attributes({predicate : Identifier(objects)})
                 else:
                     nidm_obj.add_attributes({predicate : get_RDFliteral_type(objects)})
 
@@ -904,9 +909,17 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
                 # load file
                 with open(json_source,'r+') as f:
                     json_map = json.load(f)
+            else:
+                print("ERROR: Can't open json mapping file: %s" %(json_source))
+                exit()
         except:
             # if not then it's a json structure already
             json_map = json_source
+            # added check to make sure json_map is valid dictionary
+            if not isinstance(json_map,dict):
+                print("ERROR: Invalid JSON file supplied.  Please check your JSON file with a validator first!")
+                print("exiting!")
+                exit()
 
 
     # if no JSON mapping file was specified then create a default one for variable-term mappings
@@ -954,6 +967,9 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
             except Exception as e:
                 if "list index out of range" in str(e):
                     json_key = [key for key in json_map if column.lstrip().rstrip() == key]
+                    #for key in json_map:
+                    #    print("%s == %s?" %(key,column.lstrip().rstrip()))
+
             finally:
 
                 if (json_map is not None) and (len(json_key)>0):
@@ -1032,8 +1048,10 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
                         print("Added source variable (%s) to annotations" %column)
 
                     if "isAbout" in json_map[json_key[0]]:
-                        column_to_terms[current_tuple]['isAbout'] = json_map[json_key[0]]['isAbout']
-                        print("isAbout: %s" % column_to_terms[current_tuple]['isAbout'])
+                        column_to_terms[current_tuple]['isAbout'] = {}
+                        for isabout_key,isabout_value in json_map[json_key[0]]['isAbout'].items():
+                            column_to_terms[current_tuple]['isAbout'][isabout_key] = json_map[json_key[0]]['isAbout'][isabout_key]
+                            print("isAbout: %s = %s" %(isabout_key, column_to_terms[current_tuple]['isAbout'][isabout_key]))
                     else:
                         # if user ran in mode where they want to associate concepts
                         if associate_concepts:
@@ -1075,7 +1093,9 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
             column_to_terms[subjid_tuple]['description'] = "subject/participant identifier"
             column_to_terms[subjid_tuple]['source_variable'] = str(search_term)
             column_to_terms[subjid_tuple]['valueType'] = URIRef(Constants.XSD["string"])
-            column_to_terms[subjid_tuple]['url'] =  Constants.NIDM_SUBJECTID.uri
+            column_to_terms[subjid_tuple]['isAbout'] = {}
+            column_to_terms[subjid_tuple]['isAbout']['url'] = Constants.NIDM_SUBJECTID.uri
+            column_to_terms[subjid_tuple]['isAbout']['label'] = Constants.NIDM_SUBJECTID.localpart
             # column_to_terms[subjid_tuple]['variable'] = str(column)
 
             # delete temporary current_tuple key for this variable as it has been statically mapped to NIDM_SUBJECT
@@ -1084,7 +1104,7 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
             print("Variable %s automatically mapped to participant/subject idenfier" %search_term)
             print("Label: %s" %column_to_terms[subjid_tuple]['label'])
             print("Description: %s" %column_to_terms[subjid_tuple]['description'])
-            print("Url: %s" %column_to_terms[subjid_tuple]['url'])
+            #print("Url: %s" %column_to_terms[subjid_tuple]['url'])
             print("Source Variable: %s" % column_to_terms[subjid_tuple]['source_variable'])
             print("---------------------------------------------------------------------------------------")
             continue
@@ -1140,7 +1160,7 @@ def map_variables_to_terms(df,directory, assessment_name, output_file=None,json_
 
             # now store the url from Interlex for new personal data element in column_to_terms annotation
             column_to_terms[current_tuple]['url'] = ilx_output.iri
-        except:
+        except Exception as e:
             print("WARNING: WIP: Data element not submitted to InterLex.  ")
     # write annotations to json file since data element annotations are complete
     write_json_mapping_file(column_to_terms, output_file, bids)
@@ -1336,7 +1356,9 @@ def find_concept_interactive(source_variable, current_tuple, source_variable_ann
         else:
             # user selected one of the existing concepts to add its URL to the isAbout property
             # added labels to these isAbout urls for easy querying later
-            source_variable_annotations[current_tuple]['isAbout'] = {search_result[search_result[selection]]['preferred_url'] : search_result[search_result[selection]]['label']}
+            source_variable_annotations[current_tuple]['isAbout'] = {}
+            source_variable_annotations[current_tuple]['isAbout']['url'] = search_result[search_result[selection]]['preferred_url']
+            source_variable_annotations[current_tuple]['isAbout']['label'] = search_result[search_result[selection]]['label']
             print("\nConcept annotation added for source variable: %s" %source_variable)
             go_loop = False
 
@@ -1503,6 +1525,39 @@ def annotate_data_element(source_variable, current_tuple, source_variable_annota
         print("levels: %s" % source_variable_annotations[current_tuple]['levels'])
     print("---------------------------------------------------------------------------------------")
 
+def DD_UUID (element,dd_struct):
+    '''
+    This function will produce a hash of the data dictionary (personal data element) properties defined
+    by the user for use as a UUID.  The data dictionary key is a tuple identifying the file and variable
+    name within that file to be encoded with a UUID.  The idea is that if the data dictionaries for a
+    personal data element precisely match then the same UUID will be generated.
+    :param element: element in dd_struct to create UUID for within the dd_struct
+    :param dd_struct: data dictionary json structure
+    :return: hash of
+    '''
+
+    # evaluate the compound data dictionary key and loop over the properties
+    key_tuple = eval(element)
+
+    property_string=""
+    for key, value in dd_struct[str(key_tuple)].items():
+        if key == 'label':
+            property_string = property_string + str(value)
+        if (key == 'levels') or (key == 'Levels'):
+            property_string = property_string + str(value)
+        if key == 'valueType':
+            property_string = property_string + str(value)
+        if key == 'hasUnit':
+            property_string = property_string + str(value)
+        if key == 'source_variable':
+            variable_name = value
+
+
+    crc32hash = base_repr(crc32(str(property_string).encode()), 32).lower()
+    niiri_ns = Namespace(Constants.NIIRI)
+    cde_id = URIRef(niiri_ns + safe_string(variable_name) + "_" + str(crc32hash))
+    return cde_id
+
 def DD_to_nidm(dd_struct):
     '''
 
@@ -1546,23 +1601,12 @@ def DD_to_nidm(dd_struct):
                 # cde_id = item_ns[str(key_num).zfill(4)]
 
                 # hash the key_tuple (e.g. DD(source=[FILENAME],variable=[VARNAME]))
-                crc32hash = base_repr(crc32(str(key).encode()),32).lower()
+                #crc32hash = base_repr(crc32(str(key).encode()),32).lower()
                 # md5hash = hashlib.md5(str(key).encode()).hexdigest()
 
-                # added to address some weird bug in rdflib where if the uuid starts with a number, everything up until the first
-                # alpha character becomes a prefix...
 
-                #if not (re.match("^[a-fA-F]+.*", md5hash)):
-
-                # if first digit is not a character than replace it with a randomly selected hex character (a-f).
-                #    uid_temp = md5hash
-                #    randint = random.randint(0,5)
-                #    md5hash = string.ascii_lowercase[randint] + uid_temp[1:]
-
-
-                #cde_id = item_ns[md5hash
-                #cde_id = URIRef(niiri_ns + safe_string(item) + "_" + str(md5hash))
-                cde_id = URIRef(niiri_ns + safe_string(item) + "_" + str(crc32hash))
+                cde_id = DD_UUID(key,dd_struct)
+                #cde_id = URIRef(niiri_ns + safe_string(item) + "_" + str(crc32hash))
                 g.add((cde_id,RDF.type, Constants.NIDM['DataElement']))
                 g.add((cde_id,RDF.type, Constants.PROV['Entity']))
 
@@ -1587,17 +1631,20 @@ def DD_to_nidm(dd_struct):
                 # added by DBK for multiple isAbout URLs and storing the labels along with URLs
                 # first get a uuid has for the isAbout collection for this we'll use a hash of the isAbout list
                 # as a string
-                crc32hash = base_repr(crc32(str(value).encode()), 32).lower()
+                #crc32hash = base_repr(crc32(str(value).encode()), 32).lower()
                 # now create the collection and for each isAbout create an entity to add to collection with
                 # properties for label and url
                 #g.add((isabout_collection_id, RDF.type, Constants.PROV['Collection']))
                 # for each isAbout entry, create new prov:Entity, store metadata and link it to the collection
                 for isabout_key, isabout_value in value.items():
-                    # add isAbout key which is the url
-                    g.add((cde_id, Constants.NIDM['isAbout'], URIRef(isabout_key)))
-                    # now add another entity to contain the label
-                    g.add((URIRef(isabout_key), RDF.type,Constants.PROV['Entity']))
-                    g.add((URIRef(isabout_key), Constants.RDFS['label'], Literal(isabout_value)))
+                    if (isabout_key == '@id') or (isabout_key == 'url'):
+                        last_id = isabout_value
+                        # add isAbout key which is the url
+                        g.add((cde_id, Constants.NIDM['isAbout'], URIRef(isabout_value)))
+                    elif isabout_key == 'label':
+                        # now add another entity to contain the label
+                        g.add((URIRef(last_id), RDF.type,Constants.PROV['Entity']))
+                        g.add((URIRef(last_id), Constants.RDFS['label'], Literal(isabout_value)))
 
             elif key == 'valueType':
                 g.add((cde_id, Constants.NIDM['valueType'], URIRef(value)))
@@ -1672,6 +1719,7 @@ def addGitAnnexSources(obj, bids_root, filepath = None):
             # add to graph uuid
             obj.add_attributes({Constants.PROV["Location"]: URIRef(source)})
 
+
         return len(sources)
     except Exception as e:
         if "No annex found at" not in str(e):
