@@ -200,7 +200,7 @@ def addimagingsessions(bids_layout,subject_id,session,participant, directory,img
     :param img_session:
     :return:
     '''
-    for file_tpl in bids_layout.get(subject=subject_id, session=img_session, extensions=['.nii', '.nii.gz']):
+    for file_tpl in bids_layout.get(subject=subject_id, session=img_session, extension=['.nii', '.nii.gz']):
         # create an acquisition activity
         acq=MRAcquisition(session)
 
@@ -343,9 +343,9 @@ def addimagingsessions(bids_layout,subject_id,session,participant, directory,img
                             acq_obj.add_attributes({BIDS_Constants.json_keys[key.replace(" ", "_")]:json_data.info[key]})
             # get associated events TSV file
             if 'run' in file_tpl.entities:
-                events_file = bids_layout.get(subject=subject_id, extensions=['.tsv'],modality=file_tpl.entities['datatype'],task=file_tpl.entities['task'],run=file_tpl.entities['run'])
+                events_file = bids_layout.get(subject=subject_id, extension=['.tsv'],modality=file_tpl.entities['datatype'],task=file_tpl.entities['task'],run=file_tpl.entities['run'])
             else:
-                events_file = bids_layout.get(subject=subject_id, extensions=['.tsv'],modality=file_tpl.entities['datatype'],task=file_tpl.entities['task'])
+                events_file = bids_layout.get(subject=subject_id, extension=['.tsv'],modality=file_tpl.entities['datatype'],task=file_tpl.entities['task'])
             # if there is an events file then this is task-based so create an acquisition object for the task file and link
             if events_file:
                 #for now create acquisition object and link it to the associated scan
