@@ -18,6 +18,10 @@ from prov.model import ProvAgent
 
 REST_TEST_FILE = './agent.ttl'
 BRAIN_VOL_FILES = ['./cmu_a.nidm.ttl', './caltech.nidm.ttl']
+OPENNEURO_FILES = ['ds000168.nidm.ttl']
+OPENNEURO_PROJECT_URI = None
+OPENNEURO_SUB_URI = None
+
 test_person_uuid = ""
 test_p2_subject_uuids = []
 cmu_test_project_uuid = None
@@ -25,7 +29,7 @@ cmu_test_subject_uuid = None
 
 @pytest.fixture(scope="module", autouse="True")
 def setup():
-    global cmu_test_project_uuid, cmu_test_subject_uuid
+    global cmu_test_project_uuid, cmu_test_subject_uuid, OPENNEURO_PROJECT_URI, OPENNEURO_SUB_URI
 
     if Path(REST_TEST_FILE).is_file():
         os.remove(REST_TEST_FILE)
@@ -40,11 +44,6 @@ def setup():
             "https://raw.githubusercontent.com/dbkeator/simple2_NIDM_examples/master/datasets.datalad.org/abide/RawDataBIDS/CMU_a/nidm.ttl",
             "cmu_a.nidm.ttl"
         )
-        restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
-        projects = restParser.run(['./cmu_a.nidm.ttl'], '/projects')
-        cmu_test_project_uuid = projects[0]
-        subjects = restParser.run(['./cmu_a.nidm.ttl'], '/projects/{}/subjects'.format(cmu_test_project_uuid))
-        cmu_test_subject_uuid = subjects['uuid'][0]
 
     if not Path('./caltech.nidm.ttl').is_file():
         urllib.request.urlretrieve (
@@ -52,6 +51,23 @@ def setup():
             "caltech.nidm.ttl"
         )
 
+    restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
+    projects = restParser.run(BRAIN_VOL_FILES, '/projects')
+    cmu_test_project_uuid = projects[0]
+    subjects = restParser.run(BRAIN_VOL_FILES, '/projects/{}/subjects'.format(cmu_test_project_uuid))
+    cmu_test_subject_uuid = subjects['uuid'][0]
+
+
+    if not Path('./ds000168.nidm.ttl').is_file():
+        urllib.request.urlretrieve (
+            "https://raw.githubusercontent.com/dbkeator/simple2_NIDM_examples/master/datasets.datalad.org/openneuro/ds000168/nidm.ttl",
+            "ds000168.nidm.ttl"
+        )
+
+    projects2 = restParser.run(BRAIN_VOL_FILES, '/projects')
+    OPENNEURO_PROJECT_URI = projects2[0]
+    subjects = restParser.run(BRAIN_VOL_FILES, '/projects/{}/subjects'.format(OPENNEURO_PROJECT_URI))
+    OPENNEURO_SUB_URI = subjects['uuid'][0]
 
 
 def addData(acq, data):
@@ -168,16 +184,17 @@ def test_uri_project_id():
 
     # try with the real brain volume files
     restParser = RestParser()
-    result = restParser.run(BRAIN_VOL_FILES, '/projects')
+    result = restParser.run(OPENNEURO_FILES, '/projects')
     project = result[0]
-    result = restParser.run(BRAIN_VOL_FILES, '/projects/{}'.format(project))
+    result = restParser.run(OPENNEURO_FILES, '/projects/{}'.format(project))
+
 
     assert 'dctypes:title' in result
     assert 'sio:Identifier' in result
     assert 'subjects' in result
     assert len(result['subjects']['uuid']) > 2
     assert 'data_elements' in result
-    assert len(result['data_elements']['uuid']) > 2
+    assert len(result['data_elements']['uuid']) > 1
 
 
 
@@ -213,16 +230,13 @@ def test_uri_projects_subjects_id():
     global test_person_uuid
 
     restParser = RestParser()
-    if cmu_test_project_uuid:
-        project = cmu_test_project_uuid
-    else:
-        result = restParser.run(['./cmu_a.nidm.ttl'], '/projects')
-        project = result[0]
-    result = restParser.run(['./cmu_a.nidm.ttl'], '/projects/{}/subjects'.format(project))
+    result = restParser.run(OPENNEURO_FILES, '/projects')
+    project = result[0]
+    result = restParser.run(OPENNEURO_FILES, '/projects/{}/subjects'.format(project))
     subject = result['uuid'][0]
 
     uri = '/projects/{}/subjects/{}'.format(project,subject)
-    result = restParser.run(['./cmu_a.nidm.ttl'], uri)
+    result = restParser.run(OPENNEURO_FILES, uri)
 
     assert type(result) == dict
     assert result['uuid'] == subject
@@ -232,9 +246,10 @@ def test_uri_projects_subjects_id():
     all_keys = []
     for i in instruments:
         all_keys += i.keys()
-    assert 'AGE_AT_SCAN' in all_keys
+    assert 'age' in all_keys
 
-    assert len(result['derivatives']) > 0
+    # current test data doesn't ahve derivatives!
+    # assert len(result['derivatives']) > 0
 
 
 
@@ -371,6 +386,26 @@ def test_ExtremeFilters():
     assert len(details['data_elements']['uuid']) > 0
 
 
+def test_Filter_Flexibility():
+    restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
+    if cmu_test_project_uuid:
+        project = cmu_test_project_uuid
+    else:
+        projects  = restParser.run(BRAIN_VOL_FILES, '/projects')
+        project = projects[0]
+
+    synonyms = Query.GetDatatypeSynonyms(tuple(BRAIN_VOL_FILES),project, 'ADOS_MODULE')
+    real_synonyms = [x for x in synonyms if len(x) > 1]
+
+    assert len(real_synonyms) > 1
+
+    for syn in real_synonyms:
+        if ' ' in syn:
+            continue
+        details = restParser.run(BRAIN_VOL_FILES, '/projects/{}?filter=instruments.{} gt 2'.format(project, syn))
+        assert len(details['subjects']['uuid']) > 0
+        assert len(details['data_elements']['uuid']) > 0
+
 
 def test_OpenGraph():
 
@@ -522,15 +557,20 @@ def test_project_fields_deriv():
     #fv = project['field_values']
     fv = project
     assert( type( fv ) == list )
-    fields_used = set( [ i["field"] for i in fv ]  )
-    assert field in fields_used
+    fields_used = set( [ i.label for i in fv ]  )
+    assert 'Brain Segmentation Volume (mm^3)' in fields_used
 
 def test_project_fields_instruments():
     rest_parser = RestParser(verbosity_level=0)
+
+    projects = rest_parser.run(BRAIN_VOL_FILES, '/projects')
+    proj_uuid = projects[0]
+
     rest_parser.setOutputFormat(RestParser.OBJECT_FORMAT)
 
-    field = 'AGE_AT_SCAN'
-    project = rest_parser.run( BRAIN_VOL_FILES, "/projects/{}?fields={}".format(cmu_test_project_uuid, field) )
+
+    field = 'ncidb:Age'
+    project = rest_parser.run( BRAIN_VOL_FILES, "/projects/{}?fields={}".format(proj_uuid, field) )
 
     # edited by DBK to account for only field values being returned
     #assert( 'field_values' in project )
@@ -538,7 +578,7 @@ def test_project_fields_instruments():
     #fv = project['field_values']
     fv = project
     assert( type( fv ) == list )
-    fields_used = set( [ i["field"] for i in fv ]  )
+    fields_used = set( [ i.label for i in fv ]  )
     assert field in fields_used
 
 
@@ -550,13 +590,11 @@ def test_project_fields_not_found():
     field = 'not_real_field'
     project = rest_parser.run( BRAIN_VOL_FILES, "/projects/{}?fields={}".format(cmu_test_project_uuid, field) )
 
-    # edited by DBK to account for only field values being returned
-    # assert( 'field_values' in project )
-    assert (len(project) == 0)
-    # fv = project['field_values']
-    fv = project
-    assert( type( fv ) == list )
-    assert len(fv) == 0
+
+    print (project)
+    keys = set( [ i for i in project ]  )
+
+    assert "error" in keys
 
 
 
